#include "soaring.h"

double pround(double val, int n)
{
	int ival;
	double dec = 1.0;

	while(n>0) {
		dec *= 10.0;
		n--;
	}
	if (val < 0) {
		ival = (int)(val*dec - .5);
	} else {
		ival = (int)(val*dec + .5);
	}
	return(((double)ival/dec));
}

static CharPtr not_implemented() 
{
    static char buf[10];
    StrCopy(buf,"NOI\0");
    return buf;
}

static CharPtr not_valid() 
{
    static char buf[10];
    StrCopy(buf,"NOV\0"); 
    return buf;
}

double CharToDbl(char input)
{
    switch (input) {
      case '0':
	return 0.0;
      case '1':
	return 1.0;
      case '2':
	return 2.0;
      case '3':
	return 3.0;
      case '4':
	return 4.0;
      case '5':
	return 5.0;
      case '6':
	return 6.0;
      case '7':
	return 7.0;
      case '8':
	return 8.0;
      case '9':
	return 9.0;
    }
    return 0.0;
}
	      

double StrToDbl(CharPtr input)
{
	double result=0;
	double factor=0.1;
	double negate=1.0;
	int x=0;
	if (input == 0)
		return 0.0;
	if (input[x]=='-') {
		negate=-1.0;
		x++;
	}
	while (input[x] != 0 && input[x] != '.') {
		result*=10;
		result+=CharToDbl(input[x]);
		x++;
	}
	if (input[x] == '.') {
		x++;
		while (input[x] != 0 ) {
			result+=(factor * CharToDbl(input[x]));
			factor/=10;
			x++;
		}
	}
	return (result*negate);
}

CharPtr DblToStr(double input, int prec) 
{
	int x;
	int delta;
	UInt length;
	static char result[100];
	for(x=prec;x>0;x--) {
		input*=10.0;
	}
	StrIToA(result,(int)input);
	if (prec != 0){
		length=StrLen(result);
		delta = (prec-length)+1;
		if (delta > 0) {
			for(x=length-1;x>=0;x--) {
				result[x+delta]=result[x];
			}
			for(x=0;x<delta;x++) {
				result[x]='0';
			}
			result[length+delta]=0;
			length=StrLen(result);
		}
		result[length+1]=0;
		for(x=prec;x>0;x--) {
			result[length]=result[length-1];
			length--;
		}
		result[length]='.';
	} 
	return result;
}

double next_comma_dbl(CharPtr * s)
{
    CharPtr cur;
    int x=0;
    double ret;
    cur=*s;
    while ((cur[x] != ',') && (cur[x] != 0)) {
	x++;
    }
    cur[x]=0;
    ret=StrToDbl(*s);
    *s=&(cur[x+1]);
    return ret;
}

void field_set_value(UInt fieldID, CharPtr s)
{
    FormPtr frm;
    Word    index;
    FieldPtr field;
    frm = FrmGetFormPtr(data.application.form_id);
    if (frm == NULL)
	return;
    index = FrmGetObjectIndex(frm,fieldID);
    if (index < 0) 
	return;
    field = FrmGetObjectPtr(frm,index);
    if (field == NULL)
	return;
    FldDelete(field,0,FldGetTextLength(field)),
    FldInsert(field,s,StrLen(s));
}

double field_get_double(UInt fieldID)
{
    FormPtr frm;
    Word    index;
    FieldPtr field;
    CharPtr field_str;
    frm = FrmGetFormPtr(data.application.form_id);
    index = FrmGetObjectIndex(frm,fieldID);
    field = FrmGetObjectPtr(frm,index);
    field_str = FldGetTextPtr(field);
    return StrToDbl(field_str);
}
Long field_get_long(UInt fieldID)
{
    FormPtr frm;
    Word    index;
    FieldPtr field;
    CharPtr field_str;
    frm = FrmGetFormPtr(data.application.form_id);
    index = FrmGetObjectIndex(frm,fieldID);
    field = FrmGetObjectPtr(frm,index);
    field_str = FldGetTextPtr(field);
    return StrAToI(field_str);
}
CharPtr field_get_str(UInt fieldID)
{
    FormPtr frm;
    Word    index;
    FieldPtr field;
    CharPtr field_str;
    frm = FrmGetFormPtr(data.application.form_id);
    index = FrmGetObjectIndex(frm,fieldID);
    field = FrmGetObjectPtr(frm,index);
    field_str = FldGetTextPtr(field);
    return (field_str);
}
Boolean ctl_get_value(UInt ctlID)
{
    FormPtr frm;
    Word index;
    Boolean ctl_value;
    ControlPtr ctl;
    frm = FrmGetFormPtr(data.application.form_id);
    index = FrmGetObjectIndex(frm, ctlID);
    ctl = FrmGetObjectPtr(frm, index);
    ctl_value = CtlGetValue(ctl);
    return(ctl_value);
}

void ctl_set_value(UInt ctlID, Boolean ctl_value)
{
    FormPtr frm;
    Word index;
    ControlPtr ctl;
    frm = FrmGetFormPtr(data.application.form_id);
    index = FrmGetObjectIndex(frm, ctlID);
    ctl = FrmGetObjectPtr(frm, index);
    CtlSetValue(ctl, ctl_value);
    return;
}

CharPtr print_horizontal_speed(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value * data.config.unitsconst,0);
}

CharPtr print_vertical_speed(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value* data.config.airmassconst,
		    data.config.airmassprec);
}

CharPtr print_distance(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
   return DblToStr(pround(val.value*data.config.unitsconst,0),0);
}

CharPtr print_altitude(double val)
{
    return DblToStr(pround(val * data.config.altconst,0),0);
}

//CharPtr print_altitude(DataValue val)
//{
//	if (val.valid == NOT_VALID)
//		return not_valid();
//	return DblToStr(pround(val.value* data.config.altconst,0),0);
//}

CharPtr print_temperature(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value,0);
}

CharPtr print_qnh(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value,2);
}

CharPtr print_direction(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(pround(val.value,0),0);
}

CharPtr print_longitude(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value,3);
}

CharPtr print_latitude(DataValue val)
{
    if (val.valid == NOT_VALID)
	return not_valid();
    return DblToStr(val.value,3);
}

void set_unit_constants()
{
	switch (data.config.units)
	{
		case STATUTE:
			data.config.unitsconst = MPHCONST;
			data.config.airmassconst = AIRFPMCONST;
			data.config.airmassincr = AIRFPMINCR; 
			data.config.altconst = ALTMPHNAUCONST;
			StrCopy(data.config.unitstext, "mph"); 
			StrCopy(data.config.distext, "mi");
			StrCopy(data.config.alttext, "ft");
			StrCopy(data.config.airmasstext, "fpm");
			data.config.airmassprec = 0;
			break;
		case NAUTICAL:
			data.config.unitsconst = NAUCONST;
			data.config.airmassconst = AIRKTSCONST;
			data.config.airmassincr = AIRKTSINCR; 
			data.config.altconst = ALTMPHNAUCONST;
			StrCopy(data.config.unitstext, "kts");
			StrCopy(data.config.distext, "nm");
			StrCopy(data.config.alttext, "ft");
			StrCopy(data.config.airmasstext, "kts");
			data.config.airmassprec = 1;
			break;
		case METRIC:
			data.config.unitsconst = METCONST;
			data.config.airmassconst = AIRMPSCONST;
			data.config.airmassincr = AIRMPSINCR; 
			data.config.altconst = ALTMETCONST;
			StrCopy(data.config.unitstext, "kph");
			StrCopy(data.config.distext, "km");
			StrCopy(data.config.alttext, "mt");
			StrCopy(data.config.airmasstext, "m/s");
			data.config.airmassprec = 2;
			break;
		default:
			break;
	}
}
// returns n'th (0-based) comma-delimeted field within buffer
// true if field found, false otherwise
Boolean GetField(const char *buffer, UInt n, char *result)
{
	int i;
	// skip n commas
	for (i = 0; i < n; i++) {
		while (*buffer && *buffer != ',')
			buffer++;
		if (*buffer == '\0')
			return false;
		buffer++;
	}
	while (*buffer) {
		*result++ = *buffer++;
		if (*buffer == ',' || *buffer == '*') {
			break;
		}
	}
	if (*buffer != ',' && *buffer != '*')
		return false;
	*result = '\0';
	return true;
}


/*********************************************************************************
 * SecondsToDateOrTimeString
 *    return a string of the date in HH:MM:SS or DD:MM:YY format
 * Parameters:
 *   ULong date     - date in seconds
 *   char  *textP - string to store the date in
 *   int   dttype - format of data to return 
 *                  0 for date(DD:MM:YY) 1 for time(HH:MM:SS)
 *********************************************************************************/
char* SecondsToDateOrTimeString(ULong	date, char	*textP, int dttype)
{
	DateTimeType	dtt;

	if (textP == NULL)
		return textP;

	// convert to a date time struct
	TimSecondsToDateTime(date, &dtt);

	if (dttype == 1) {
		// the hour
		textP[0] = (dtt.hour / 10) + '0';
		textP[1] = (dtt.hour % 10) + '0';

		textP[2] = ':';
	
		// the minute
		textP[3] = (dtt.minute / 10) + '0';
		textP[4] = (dtt.minute % 10) + '0';

		textP[5] = ':';

		// the second
		textP[6] = (dtt.second / 10) + '0';
		textP[7] = (dtt.second % 10) + '0';
	} else {
		// the month
		textP[0] = (dtt.day / 10) + '0';
		textP[1] = (dtt.day % 10) + '0';

		textP[2] = ':';
	
		// the day
		textP[3] = (dtt.month / 10) + '0';
		textP[4] = (dtt.month % 10) + '0';

		textP[5] = ':';

		// the year
		textP[6] = (dtt.year / 10) + '0';
		textP[7] = (dtt.year % 10) + '0';
	}

	textP[8] = '\0';

	return textP;
}

/*****************************************************************************
 * StringToDateOrTime
 *    return a date/time structure with the date or time info filled in
 * Parameters:
 *   Char  textP - string containing the date or time info
 *   DateTimeType dtt  - date/time structure to fill in
 *   int dttype - 0 for date(DDMMYY), 1 for time(HHMMSS)
 *****************************************************************************/
void StringToDateOrTime(CharPtr textP, DateTimeType *dtt, int dttype)
{
	char tempchar[4];
	int int1, int2, int3;

	tempchar[0]=textP[0];
	tempchar[1]=textP[1];
	tempchar[2]='\0';
	int1=StrAToI(tempchar);
	tempchar[0]=textP[2];
	tempchar[1]=textP[3];
	tempchar[2]='\0';
	int2=StrAToI(tempchar);
	tempchar[0]=textP[4];
	tempchar[1]=textP[5];
	tempchar[2]='\0';
	int3=StrAToI(tempchar);
	if (dttype == 1) {
		dtt->hour=int1;
		dtt->minute=int2;
		dtt->second=int3;
	} else {
		dtt->day=int1;
		dtt->month=int2;
		dtt->year=int3;
//		dtt->year=2000+int3-1904;
	}
	return;
}
